#include-once

; #INDEX# =======================================================================================================================
; Title .........: CODABAR barcode generator
; AutoIt Version : 3.3.12.0
; Description ...: Create a CODABAR barcode from supplied data
; Author(s) .....: David Williams (willichan)
; Dll ...........:
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;_BcodeCODABAR_GenCode
; ===============================================================================================================================

; #INTERNAL_USE_ONLY# ===========================================================================================================
;__BcodeCODABAR_BitmapSetResolution
;__BcodeCODABAR_CreateImage
;__BcodeCODABAR_DataInit
;__BcodeCODABAR_EncodeData
;__BcodeCODABAR_StartStop
; ===============================================================================================================================

; #VARIABLES# ===================================================================================================================
; ===============================================================================================================================

; #CONSTANTS# ===================================================================================================================
; ===============================================================================================================================

#include <GDIPlus.au3>
#include "StringSize.au3"; by Melba23 - http://www.autoitscript.com/forum/topic/114034-stringsize-m23-new-version-16-aug-11/
#include <File.au3>

; #FUNCTION# ====================================================================================================================
; Name ..........: _BcodeCODABAR_GenCode
; Description ...: Creates a Code39 barcode from supplied data
; Syntax ........: _BcodeCODABAR_GenCode($sData[, $sOutFile = Default[, $iDPI = Default[, $fMinBar = Default[,
;                  $fBarHeight = Default[, $iPrintText = Default]]]]])
; Parameters ....: $sData               - A string value to encode into a Code39 barcode
;                  $sStartStop          - A two character string containing the start and stop codes to use. Default is "AB".
;                                         Can use ABCD or TN*E notation
;                  $sOutFile            - [optional] Where to write out the BMP file. Default is to create a random temp file
;                                         0 = Copy to the clipboard (planned, but not yet implemented)
;                                         1 = Create a randomly named temp file (Default)
;                                         String = Write to specified path\filename (assumes valid path and filename)
;                  $iDPI                - [optional] The dots-per-inch setting for the BMP file.  Default is 96
;                  $fMinBar             - [optional] Width of the narrowest bar, in inches (0.01 is the recommended standard for
;                                         most handheld scanners). Default is 0.01 inches
;                  $fBarHeight          - [optional] The height of the barcode, in inches (15% of the width or 0.25 inches
;                                         ,whichever is larger, is the standard). Default is to automatically calculate according
;                                         to this standard.
;                  $iPrintText          - [optional] Whether to print the data string under the barcode. Default is True.
; Return values .: String containing the location and name of the BMP file created, or "" if copied to the clipboard.
; Author ........: David E Williams (willichan)
; Modified ......:
; Remarks .......: Invalid characters in $sData will be skipped rather than generating an error
;                  Most scanners will only recognize numbers.  Using the extended characters (-$:/.+) can confuse some readers.
;                  Copying to the clipboard is not yet implemented.  Random temp file will be made instead.
; Required ......: http://www.autoitscript.com/forum/topic/114034-stringsize-m23-new-version-16-aug-11/
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _BcodeCODABAR_GenCode($sData, $sStartStop = Default, $sOutFile = Default, $iDPI = Default, $fMinBar = Default, $fBarHeight = Default, $iPrintText = Default)
	If (not IsString($sData)) Or (StringLen($sData) < 1) Then Return ""
	Local $vBarcodeObj[7], $t
	If $sStartStop = Default Then $sStartStop = "AB"
	If StringLen($sStartStop)<>2 Then Return ""
	$sStartStop = StringUpper($sStartStop)
	If IsNumber($sOutFile) Then $sOutFile = 1 ;;; *** treat any number like a temp file, in case the user messed up ***
	If ($sOutFile = Default) Or ($sOutFile = 1) Then $sOutFile = _TempFile(@TempDir, "~", ".bmp", 7)
	If $iPrintText = Default Then $iPrintText = True ;print text at bottom of barcode
	If $iDPI = Default Then $iDPI = 96 ;Dots Per Inch used for the image file
	If $fMinBar = Default Then $fMinBar = 0.01 ;Minimum bar width in Inches
	Local $iModWidth = Ceiling($iDPI * $fMinBar) ;Number of Dots per Minimum Bar
	$vBarcodeObj[1] = $iModWidth
	$vBarcodeObj[2] = $sOutFile
	$vBarcodeObj[3] = $iDPI
	$vBarcodeObj[6] = $sStartStop
	$t = __BcodeCODABAR_EncodeData($sData, $vBarcodeObj)
	If $iPrintText Then
		$vBarcodeObj[0] = $t[0] ;encoded text
	Else
		$vBarcodeObj[0] = ""
	EndIf
	$vBarcodeObj[5] = $t[1] ;bar encoding
	If $fBarHeight = Default Then
		$fBarHeight = ((StringLen($vBarcodeObj[5]) * $iModWidth) / $iDPI) * 0.15
		If $fBarHeight < 0.25 Then $fBarHeight = 0.25
		$fBarHeight = Ceiling($iDPI * $fBarHeight)
	EndIf
	$vBarcodeObj[4] = $fBarHeight
	__BcodeCODABAR_CreateImage($vBarcodeObj)
EndFunc   ;==>_BcodeCODABAR_GenCode

Func __BcodeCODABAR_BitmapSetResolution($hBitmap, $nDpiX, $nDpiY)
	Local $aResult = DllCall($__g_hGDIPDll, "uint", "GdipBitmapSetResolution", "handle", $hBitmap, "float", $nDpiX, "float", $nDpiY)
	If @error Then Return SetError(@error, @extended, False)
	Return $aResult[0] = 0
EndFunc   ;==>__BcodeCODABAR_BitmapSetResolution

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __BcodeCODABAR_CreateImage
; Description ...: converts bar/space data into a BMP
; Syntax ........: __BcodeCODABAR_CreateImage(Byref $vBarcodeObj)
; Parameters ....: $vBarcodeObj         - Array containing barcode data
; Return values .: None
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __BcodeCODABAR_CreateImage(ByRef $vBarcodeObj)
	Local $i, $cx
	Local Const $iFontMargin = 14
	;;create image
	_GDIPlus_Startup()
	Local $hBitmap = _GDIPlus_BitmapCreateFromScan0((StringLen($vBarcodeObj[5]) + 14) * $vBarcodeObj[1], $vBarcodeObj[4] + $iFontMargin, $GDIP_PXF01INDEXED)
	__BcodeCODABAR_BitmapSetResolution($hBitmap, $vBarcodeObj[3], $vBarcodeObj[3])
	Local $sCLSID = _GDIPlus_EncodersGetCLSID("bmp")
	Local $hBmpCtxt = _GDIPlus_ImageGetGraphicsContext($hBitmap)
	_GDIPlus_GraphicsSetSmoothingMode($hBmpCtxt, 0)
	_GDIPlus_GraphicsClear($hBmpCtxt, 0xFFFFFFFF)
	;;draw lines
	For $i = 1 To StringLen($vBarcodeObj[5])
		If StringMid($vBarcodeObj[5], $i, 1) = 1 Then
			$cx = (($i - 1) + 7) * $vBarcodeObj[1]
			_GDIPlus_GraphicsFillRect($hBmpCtxt, $cx, 0, $vBarcodeObj[1], $vBarcodeObj[4], 0)
			If @error Then ConsoleWrite("unable to draw bar " & $i & @CRLF)
		EndIf
	Next

	If $vBarcodeObj[0] <> "" Then
		$i = _StringSize($vBarcodeObj[0], 8, Default, Default, "Arial")
		$cx = ((StringLen($vBarcodeObj[5]) * $vBarcodeObj[1]) / 2) - ($i[2] / 2)
		_GDIPlus_GraphicsDrawString($hBmpCtxt, $vBarcodeObj[0], $cx, $vBarcodeObj[4] + 1, "Arial", 8)
		If @error Then ConsoleWrite("unable to draw text - " & @extended & @CRLF)
	EndIf

	;;save and dispose of image
	_GDIPlus_ImageSaveToFile($hBitmap, $vBarcodeObj[2])
	If @error Then ConsoleWrite("unable to write to file" & $vBarcodeObj[2] & @CRLF)
	_GDIPlus_GraphicsDispose($hBmpCtxt)
	_GDIPlus_BitmapDispose($hBitmap)
	_GDIPlus_Shutdown()
EndFunc   ;==>__BcodeCODABAR_CreateImage

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __BcodeCODABAR_DataInit
; Description ...: loads array with coding data
; Syntax ........: __BcodeCODABAR_DataInit(Byref $aEncoding)
; Parameters ....: $aEncoding           - variable to fill with coding data array
; Return values .: None
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __BcodeCODABAR_DataInit(ByRef $aEncoding)
				;;;Char, Bar Pattern
			Local $aEnc[16][2] = [ _
				['0','10101000111'], _
				['1','10101110001'], _
				['2','10100010111'], _
				['3','11100010101'], _
				['4','10111010001'], _
				['5','11101010001'], _
				['6','10001010111'], _
				['7','10001011101'], _
				['8','10001110101'], _
				['9','11101000101'], _
				['-','10100011101'], _
				['$','10111000101'], _
				[':','1110101110111'], _
				['/','1110111010111'], _
				['.','1110111011101'], _
				['+','10111000111000111'] _
			]
			;last four are start/stop codes only
	$aEncoding = $aEnc
	$aEnc = 0
EndFunc


; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __BcodeCODABAR_EncodeData
; Description ...: encodes data into bar/space code for generating barcode
; Syntax ........: __BcodeCODABAR_EncodeData($sData, Byref $vBarcodeObj)
; Parameters ....: $sData               - data to be converted to a barcode
;                  $vBarcodeObj         - array containing barcode object information
; Return values .: array contining encoded data
;                      [0] - text for placing at the bottom of the barcode
;                      [1] - bar/space coding to generate barcode
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __BcodeCODABAR_EncodeData($sData, ByRef $vBarcodeObj)
	Local $aReturn[2] = [StringLeft($vBarcodeObj[6],1) & " ", __BcodeCODABAR_StartStop(StringLeft($vBarcodeObj[6],1)) & "0"]
	Local $aEncoding, $iTmp

	__BcodeCODABAR_DataInit($aEncoding)

	Local $iLoop
	For $iLoop = 1 to StringLen($sData)
		$iTmp = StringMid($sData, $iLoop, 1)
		Switch $iTmp
			Case '0' to '9'
				;do nothing
			Case '-'
				$iTmp = 10
			Case '$'
				$iTmp = 11
			Case ':'
				$iTmp = 12
			Case '/'
				$iTmp = 13
			Case '.'
				$iTmp = 14
			Case '+'
				$iTmp = 15
			Case Else
				ContinueLoop
		EndSwitch
		$aReturn[0] &= $aEncoding[$iTmp][0]
		$aReturn[1] &= $aEncoding[$iTmp][1] & "0"
	Next
	$aReturn[0] &= " " & StringRight($vBarcodeObj[6],1)
	$aReturn[1] &= __BcodeCODABAR_StartStop(StringRight($vBarcodeObj[6],1))
	$aEncoding = 0
	Return $aReturn
EndFunc   ;==>__BcodeCODABAR_EncodeData

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name ..........: __BcodeCODABAR_StartStop
; Description ...: Encode Start/Stop code
; Syntax ........: __BcodeCODABAR_StartStop($sChar)
; Parameters ....: $sChar               - Start/Stop character to encode
; Return values .: string representing the encoded Start/Stop character
;                  an invalid character returns an empty string
; Author ........: willichan
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func __BcodeCODABAR_StartStop($sChar)
	Switch StringLeft($sChar, 1)
		Case "A", "T"
			Return '1011100010001'
		Case "B", "N"
			Return '1000100010111'
		Case "C", "*"
			Return '1010001000111'
		Case "D", "E"
			Return '1010001110001'
		Case Else
			Return ""
	EndSwitch
EndFunc
